%define default_toolprefix /export/tools
%{?toolprefix:%define default_toolprefix %{toolprefix}}

%define default_cross_os linux
%{?cross_os:%define default_cross_os %{cross_os}}

%define default_codename %{nil}
%{?codename:%define default_codename %{codename}}

%define default 0
%{?_with_default: %{expand: %%define default 1}}

%define via 0
%{?_with_via: %{expand: %%define via 1}}

%define redhat_6_2_compat 0
%{?_with_redhat62compat: %{expand: %%define redhat_6_2_compat 1}}

%if %{redhat_6_2_compat}
%define GCC i386-glibc21-linux-gcc
%else
%define GCC gcc
%endif

%define do_i386_only 0
%{?_with_i386: %{expand: %%define do_i386_only 1}}
%define do_mips_only 0
%{?_with_mips: %{expand: %%define do_mips_only 1}}
%define do_mipsel_only 0
%{?_with_mipsel: %{expand: %%define do_mipsel_only 1}}

%define i386_target i386
%{?_with_i686: %{expand: %%define i386_target i686}}

%define mips_target mips
%define mipsel_target mipsel

%define build_i386 1
%define build_mips 1
%define build_mipsel 1

%if %{do_i386_only}
%define mips_target	""
%define mipsel_target	""
%define build_mips 0
%define build_mipsel 0
%endif

%if %{do_mips_only}
%define i386_target	""
%define mipsel_target	""
%define build_i386 0
%define build_mipsel 0
%endif

%if %{do_mipsel_only}
%define i386_target	""
%define mips_target	""
%define build_i386 0
%define build_mips 0
%endif

%define all_tagerts %{i386_target} %{mips_target} %{mipsel_target}

%define glibc_version 2.2.5

Summary: The GNU cross compile toolchain for Linux
Name: toolchain
Version: 20021126
Release: 2.niisi
Copyright: GPL
Group: Development/Tools
Source0: tools-%{version}.tar.bz2
Source1: tools-glibc-%{glibc_version}.tar.bz2


####Patches from cross-gcc-2.95.4


#patch18
Patch1: toolchain-mips-reg_zero.patch

#patch22
Patch2: toolchain-barrier.patch

####

Patch10: toolchain-reg_dead_after_loop.patch


%if %{redhat_6_2_compat}
BuildPrereq: compat-gcc = 6.2
BuildPrereq: compat-glibc = 6.2
%endif

Prefix: %{default_toolprefix}
Buildroot: %{_tmppath}/%{name}-%{version}-%{release}-root
Epoch: 1
ExclusiveArch: %{ix86}

%define __find_provides %{_builddir}/%{name}/find_provides.sh
%define __find_requires %{_builddir}/%{name}/find_requires.sh

%description
The common files in the GNU cross compile toolchain for Linux.

%package i386
Summary: The i386 part of the GNU cross compile toolchain
Group: Development/Tools
Requires: %{name} = %{version}-%{release}

%description i386
The i386 part of the GNU cross compile toolchain.

%package mips
Summary: The mips part of the GNU cross compile toolchain
Group: Development/Tools
Requires: %{name} = %{version}-%{release}

%description mips
The mips part of the GNU cross compile toolchain.

%package mipsel
Summary: The mipsel part of the GNU cross compile toolchain
Group: Development/Tools
Requires: %{name} = %{version}-%{release}

%description mipsel
The mipsel part of the GNU cross compile toolchain.

%prep
%setup -q -n %{name} -c -a 1

%patch1
%patch2

%patch10

%if "%{default_codename}" != ""
for v in gcc/version.c gcc/f/version.c; do
  perl -pi -e 's/\";$/ %{default_codename}";/' tools-%{version}/$v
done
%endif

cat > find_provides.sh <<EOF
#!/bin/sh
arch_prefix=
export arch_prefix
read list
case \$list in
	*mips-linux*)
		arch_prefix=mips-linux;;
	*mipsel-linux*)
		arch_prefix=mipsel-linux;;
	*i386-linux*)
		arch_prefix=i386-linux;;
esac
while read file; do
	list="\$list \$file"
done
for file in \$list; do
	echo \$file
done |
/usr/lib/rpm/find-provides |
grep -v '(GLIBC_INTE' |
grep -v '(GLIBC_PRI' |
while read i; do
if [ -n "\$arch_prefix" ]; then 
	echo \$arch_prefix-\$i
else
	echo \$i
fi
done
exit 0
EOF
chmod +x find_provides.sh

cat > find_requires.sh <<EOF
#!/bin/sh
while read file; do
  case \$file in
  %{buildroot}%{prefix}/*-%{default_cross_os}/*)
    ;;
  *)
    echo \$file
    ;;
  esac
done |
/usr/lib/rpm/find-requires |
grep -v libbfd |
grep -v libopcodes
exit 0
EOF
chmod +x find_requires.sh

%build
if [ -x /usr/bin/getconf ] ; then
  numprocs=$(/usr/bin/getconf _NPROCESSORS_ONLN)
  if [ $numprocs -eq 0 ]; then
    numprocs=1 
  fi
else
  numprocs=1
fi
numprocs=`expr $numprocs + $numprocs`

build_tool()
{
  cross_arch=$1
  cross_target=$cross_arch-%{default_cross_os}
  mkdir build-$cross_target
  pushd build-$cross_target
  default_arch_flags=
%if %{default}
  case "$cross_arch" in
  i[34567]86)
%if %{via}
    if test "$cross_arch" = "i386"; then
      default_cross_arch=viac3
    else
      default_cross_arch=$cross_arch
    fi
%else
    default_cross_arch=$cross_arch
%endif
    default_arch_flags="--with-arch=$default_cross_arch \
			--with-cpu=$default_cross_arch"
    ;;
  esac
%endif
  CC=%{GCC} \
  ../tools-%{version}/configure $default_arch_flags \
	--target=$cross_target \
	--prefix=%{prefix} \
	--with-local-prefix=%{prefix} \
	--enable-threads=posix \
	--enable-haifa \
	--enable-shared \
	--disable-checking
  make -C bfd headers
  make -C gcc cstamp-h
  make -C gcc cstamp-h

  build_glibc_dir=`pwd`/$cross_target/glibc
  mkdir $build_glibc_dir
  rm -rf glibc-$cross_arch
  mkdir glibc-$cross_arch
  pushd glibc-$cross_arch
  for d in usr lib; do
    cp -af ../../glibc/$cross_arch/$d .;
  done
  cp -af usr/include usr/lib lib $build_glibc_dir
  rm -rf lib
  cd usr/lib
  for l in `find -type l`; do
    link=`file $l | grep "broken symbolic" 2> /dev/null` || true
    if [ -n "$link" ]; then
      lib=`echo $link | awk '{print $6;}'`
      rm -f $build_glibc_dir/lib/$l
      ln -s `basename $lib` $build_glibc_dir/lib/$l
    fi
  done
  dl=`readelf -d $build_glibc_dir/lib/libc.so.* | grep NEEDED | \
	sed -e "s/.*\[\(.*\)\].*/\1/"`
  sed -e "s,/lib/libc.so,$build_glibc_dir/lib/libc.so," \
    -e "s,/usr/lib/libc,$build_glibc_dir/lib/$dl /usr/lib/libc," \
    -e "s,/usr/lib/libc,$build_glibc_dir/lib/libc," \
	libc.so > $build_glibc_dir/lib/libc.so 
  sed -e "s,/lib/libpthread.so,$build_glibc_dir/lib/libpthread.so," \
      -e "s,/lib/librt.so,$build_glibc_dir/lib/librt.so," \
	librt.so > $build_glibc_dir/lib/librt.so 
  popd
  rm -rf glibc-$cross_arch

  make -j $numprocs
  popd
}

for t in %{all_tagerts}; do
  test -n "$t" || continue
  build_tool $t
done

# Build gdb info pages.
for t in %{all_tagerts}; do
  test -n "$t" || continue
  test -d build-$t-%{default_cross_os}/gdb || continue
  make info -C build-$t-%{default_cross_os}/gdb
  break
done

%install
rm -rf ${RPM_BUILD_ROOT}
for t in %{all_tagerts}; do
  test -n "$t" || continue
  cross_target=$t-%{default_cross_os}
  pushd build-$cross_target
  make install prefix=${RPM_BUILD_ROOT}%{prefix}
  build_glibc_dir=`pwd`/$cross_target/glibc
  cp -afv $build_glibc_dir/. \
	${RPM_BUILD_ROOT}%{prefix}/$cross_target
  for l in libc.so librt.so; do
    sed -e "s,$build_glibc_dir/lib/,%{prefix}/$cross_target/lib/,g" \
	$build_glibc_dir/lib/$l \
	> ${RPM_BUILD_ROOT}%{prefix}/$cross_target/lib/$l
  done
  popd
done

# Install info pages.
for t in %{all_tagerts}; do
  test -n "$t" || continue
  for d in bfd binutils gas gdb ld; do
    test -d build-$t-%{default_cross_os}/$d || continue
    make -C build-$t-%{default_cross_os}/$d \
	install-info prefix=${RPM_BUILD_ROOT}%{prefix}
  done
  break
done

%clean
rm -rf ${RPM_BUILD_ROOT}

%files
%defattr(-,root,root)
%dir %{prefix}
%dir %{prefix}/*-pc-linux
%dir %{prefix}/lib
%dir %{prefix}/lib/gcc-lib
%dir %{prefix}/man
%dir %{prefix}/man/man1
%{prefix}/include
%{prefix}/info

%if %{build_i386}
%files i386
%defattr(-,root,root)
%dir %{prefix}/*-pc-linux/i386-%{default_cross_os}
%dir %{prefix}/*-pc-linux/i386-%{default_cross_os}/lib
%{prefix}/*-pc-linux/i386-%{default_cross_os}/lib/lib*-*.so
%{prefix}/bin/i386-%{default_cross_os}-*
%{prefix}/lib/gcc-lib/i386-%{default_cross_os}
%{prefix}/man/man1/i386-%{default_cross_os}-*
%{prefix}/i386-%{default_cross_os}
%endif

%if %{build_mips}
%files mips
%defattr(-,root,root)
%dir %{prefix}/*-pc-linux/mips-%{default_cross_os}
%dir %{prefix}/*-pc-linux/mips-%{default_cross_os}/lib
%{prefix}/*-pc-linux/mips-%{default_cross_os}/lib/lib*-*.so
%{prefix}/bin/mips-%{default_cross_os}-*
%{prefix}/lib/gcc-lib/mips-%{default_cross_os}
%{prefix}/man/man1/mips-%{default_cross_os}-*
%{prefix}/mips-%{default_cross_os}
%endif

%if %{build_mipsel}
%files mipsel
%defattr(-,root,root)
%dir %{prefix}/*-pc-linux/mipsel-%{default_cross_os}
%dir %{prefix}/*-pc-linux/mipsel-%{default_cross_os}/lib
%{prefix}/*-pc-linux/mipsel-%{default_cross_os}/lib/lib*-*.so
%{prefix}/bin/mipsel-%{default_cross_os}-*
%{prefix}/lib/gcc-lib/mipsel-%{default_cross_os}
%{prefix}/man/man1/mipsel-%{default_cross_os}-*
%{prefix}/mipsel-%{default_cross_os}
%endif

%changelog

* Wed Apr 14 2004 Sergey Rogozhkin <rogozhkin@niisi.msk.ru>
- Add patches from cross-gcc-2.95.4 (patch1, patch2, patch3)
- 	patch1:  fixed a few instruction definitions to use the $0 register where a register is required but a zero constant is allowed in the MIPS machine description
-	patch2:	 fix barrier deletion in delete_insn(); by Hiroyuki Machida
- Fix reg_dead_after_loop function: it must pay attention to register utilization in REG_EQUAL and REG_EQUIV notes  (patch10)
- Update release to 2.niisi

* Mon Feb 9  2004 Sergey Rogozhkin <rogozhkin@niisi.msk.ru>
- Use correct name of libpthread.so library in %%install section (there was libbpthread.so)
- Change find_provides to show cross libraries in package provides
- Update release to 1.niisi

* Wed Nov 27 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20021126 with binutils 2.13.90.0.16
  glibc 2.2.5-42 and gcc 2.96-113. Remove gdb.

* Wed Aug 14 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020814 with binutils 2.13.90.0.4.

* Thu Aug  8 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020809 with binutils 2.13.90.0.3
  and gdb 5.2.1-0.2 (5.2.1.0.90_2002-08-09-cvs).

* Thu Aug  1 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020801 with binutils 2.13.90.0.2.

* Mon Jul 29 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020729 with glibc 2.2.5-37, gdb
  5.2.1-0.1 (5.2.1.0.90_2002-07-29-cvs), gcc 2.96-112 and binutils
  2.12.90.0.16.

* Wed Jul 17 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020717 with gdb 5.2-0.9
  (5.2.0_2002-07-17-cvs) and binutils 2.12.90.0.15.

* Thu Jun 27 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020627 with gdb 5.2-0.8
  (5.2.0_2002-06-27-cvs) and binutils 2.12.90.0.14.

* Wed Jun 26 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020626 with glibc 2.2.5-36, gdb
  5.2-0.8 (5.2.0_2002-06-26-cvs) and binutils 2.12.90.0.13.

* Wed Jun 19 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020618 with gdb 5.2-0.7
  (5.2.0_2002-06-19-cvs) and binutils 2.12.90.0.12.

* Sun May 26 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020525 with gdb 5.2-0.5
  (5.2.0_2002-05-26-cvs) and binutils 2.12.90.0.9.

* Thu May 23 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020523 with gdb 5.2-0.5
  (5.2.0_2002-05-23-cvs) and binutils 2.12.90.0.8.

* Tue Apr 23 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020423 with gdb 5.2-0.3
  (5.1.91-2002-04-23-cvs) and binutils 2.12.90.0.7.

* Sun Apr 21 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020421 with gcc 2.96-110,
  glibc 2.2.5-34, gdb 5.2-0.3 (5.1.91-2002-04-21-cvs) and binutils
  2.12.90.0.5.

* Mon Apr  8 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020408 with gcc 2.96-109,
  glibc 2.2.5-30, gdb 5.2-0.2 (5.1.90-2002-04-08-cvs) and binutils
  2.12.90.0.4.

* Tue Mar 26 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020323 with gcc 2.96-107,
  glibc 2.2.5-27, gdb 5.2-0.1 (5.1.90-2002-03-22-cvs) and binutils
  2.12.90.0.3.

* Thu Feb  7 2002 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20020207 with gdb 5.1.0.1 and
  binutils 2.11.93.0.2.

* Mon Nov 26 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20011125 with gdb 5.1 and binutils
  2.11.92.0.12.

* Sun Oct 21 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20011020 with binutils 2.11.92.0.10.

* Sat Oct 20 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20011020 with binutils 2.11.92.0.8.

* Wed Oct 17 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20011015 with gcc 2.96-99, gdb 5.0.91
  2001-10-15-cvs, glibc 2.2.4-19 and binutils 2.11.92.0.7.

* Thu Aug 30 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20010830 with binutils 2.11.90.0.31.

* Tue Aug 28 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20010827 with gdb 5.0.90 2000-08-21,
  binutils 2.11.90.0.29, glibc 2.2.4 and gcc 2.96-97.

* Fri Aug 10 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20010810 with gcc 2.96-96, gdb 5.0.90
  2000-08-10 and binutils 2.11.90.0.27.

* Thu Jul 26 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20010726 with gcc 2.96-95 and
  binutils 2.11.90.0.25.

* Wed Jul 25 2001 H.J. Lu <hjl@gnu.org>
- Add selective build.

* Sun Jul 15 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20010713 with gcc 2.96-94 and
  binutils 2.11.90.0.24.
- Add gdb 2001-07-13-cvs.

* Sun Jul  8 2001 H.J. Lu <hjl@gnu.org>
- Update the toolchain version to 20010707 with gcc 2.96-93 and
  binutils 2.11.90.0.23.
- Fix the gcc stabs assembly output.

* Sun Jun 24 2001 H.J. Lu <hjl@gnu.org>
- Initial version.
